/* file: das16s-config.c */
/* this file sends configuration/action messages to the das16s driver via ioctl */
/* steve rosenbluth JHCS 2-2000 */
/* compile with: gcc -I/usr/src/linux/include -O2 -o das16s-config das16s-config.c */
/*  epson dot-matrix printer condense code */

/*----------------------------------------------------------------
    This source file is subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
------------------------------------------------------------------*/


#include "das16s-config.h"                                 /* defs for talking to driver */
#include <sys/mman.h>                                                       /* for mlock() */

#include <sys/types.h>                                                      /* for open, flags */
#include <sys/stat.h>
#include <fcntl.h>


/****************************/
int main(int argc, char *argv[])
{
    char mode = '!';
    int fd_ctl;                                                              /* file descriptor for das16s16s device */
    int ioc_cmd = NONE;
    ctl_msg_struct_t ctl_msg;                                                /* for talking to daq driver */

    mlock(&ctl_msg, sizeof(ctl_msg_struct_t));                               /* we lock memory so our struct can be read by driver */

    if (argc > 1)
    {
        sscanf(argv[1], "%c", &mode);                                        /* convert ascii hex port value to decimal */
        if (argc > 2)
        {
            if (mode == 'f')   
                sscanf(argv[2],"%u",&ctl_msg.adc_rate_divisor);                        /* capture unsigned divisor */
            else if (mode == 'o')                                                   /* digital output */
                ctl_msg.dio_output_val = atoi(argv[2]);                        /* convert byte to output */
            else if (strncmp(argv[2], "10", 2) == 0)                        /* continuous or burst mode*/
                ctl_msg.volt_range = BI_TEN_VOLT;
            else if (strncmp(argv[2], "5", 1) == 0)
                ctl_msg.volt_range = BI_FIVE_VOLT;
            else if (strncmp(argv[2], "2.5", 3) == 0)
                ctl_msg.volt_range = BI_TWO_VOLT;
        }

        if ((fd_ctl = open("/dev/das16s", O_WRONLY)) < 0)            /* this device is for this proc to send commands to device driver */
        {
            perror("das16s-config: Error opening /dev/das16s");
            exit(1);
        }
    }
    else
    {
        printf("Usage: das16s-config [c b g s o f] [[10,5,2.5] [0-255] [2-65535]]\nc: continuous acq\nb: burst acq\ng: 'go' sample\n"
            "s: stop sampling\no: digital output\nf: set frequency divisor\nq: query driver variables\n10,5,2.5: voltage range\n"
            "0-255: digital output value\n2-65535: frequency divisor\n");
        exit(0);
    }

    if (mode == 'c')                                                           /* continuous acq */
    {
        ctl_msg.command = CONFIGURE_ACQUISITION;
        ctl_msg.conversion_mode = CONTIN_ACQUISITION;
        ctl_msg.num_chan = 16;
        ctl_msg.single_chan = 0;
        ctl_msg.num_mux_banks = 4;
        /* volt range set elsewhere */
        /*ctl_msg.result_numsys = BOB_NUMSYS;*/
        ctl_msg.error_correcting = FALSE;
        ctl_msg.aux_adc_rate_divisor = 10;
        ctl_msg.adc_rate_divisor = 20;
        /* dio_output_val not settable here */
        ctl_msg.num_dig_in_banks = 4;
        ioc_cmd = CONFIGURE_ACQUISITION/*0*/;
    }
    else if (mode == 'b')                                                      /* burst acq */
    {
        ctl_msg.command = CONFIGURE_ACQUISITION;
        ctl_msg.conversion_mode = BURST_ACQUISITION;
        ctl_msg.num_chan = 16;
        ctl_msg.single_chan = 0;
        ctl_msg.num_mux_banks = 4;
        /* volt_range set above */
        /*ctl_msg.result_numsys = BOB_NUMSYS;*/
        ctl_msg.error_correcting = FALSE;
        ctl_msg.aux_adc_rate_divisor = 10;
        ctl_msg.adc_rate_divisor = 20/*50*//*65535*/;
        /* dio_output_val not settable here */
        ctl_msg.num_dig_in_banks = 4;
        ioc_cmd = CONFIGURE_ACQUISITION/*0*/;
    }
    else if (mode == 'g')                                                      /* "go" start acq */
    {
        ctl_msg.command = START_ACQUISITION;
        ctl_msg.conversion_mode = START_ACQUISITION;
        ioc_cmd = START_ACQUISITION/*1*/;
    }
    else if (mode == 's')                                                      /* stop acq */
    {
        ctl_msg.command = STOP_ACQUISITION;
        ctl_msg.conversion_mode = STOP_ACQUISITION;
        ctl_msg.aux_adc_rate_divisor = -1;
        ctl_msg.adc_rate_divisor = -1;
        ioc_cmd = STOP_ACQUISITION/*2*/;
    }
    else if (mode == 'o')                                                      /* stop acq */
    {
        ctl_msg.command = DIGITAL_OUTPUT;
        /* value is already in ctl_msg.dio_output_val */
        ioc_cmd = DIGITAL_OUTPUT/*5*/;
    }
    else if (mode == 'f')                                                      /* freq adj */
    {
        ctl_msg.command = CONFIGURE_ACQUISITION;
        ctl_msg.conversion_mode = -1;
        ctl_msg.num_chan = -1;
        ctl_msg.single_chan = -1;
        ctl_msg.num_mux_banks = -1;
        ctl_msg.aux_adc_rate_divisor = 10;
        /* user supplied divisor is already in ctl_msg.adc_rate_divisor */
        /*ctl_msg.result_numsys = -1;*/
        ctl_msg.error_correcting = -1;
        ctl_msg.num_dig_in_banks = -1;
        ioc_cmd = CONFIGURE_ACQUISITION/*4*/;
    }
    else if (mode == 'q')
    {
        ctl_msg.command = -1;
        ctl_msg.conversion_mode = -1;
        ioc_cmd = QUERY_ACQUISITION;    /*6*/
    }
    else
        ioc_cmd = NONE;

    if (ioc_cmd != NONE)
    {
        /*printf("addr of our struct is %p\n",&ctl_msg);*/

        if (ioctl(fd_ctl, ioc_cmd, &ctl_msg) < 0)                          /* write startup message to device */
            perror("das16s-config: Error writing file descriptor");

        /*printf("fake ioctl: cmd = %d\n",ioc_cmd);*/
    }

    /* printf("\nSent message to daqcard700 ...\ncommand: %d, conversion_mode: %d, num_chan: %d, single_chan: %d, num_adc_banks: %d,
     * timer_period: %d\n",
     * ctl_msg.command,ctl_msg.conversion_mode,ctl_msg.num_chan,ctl_msg.single_chan,ctl_msg.num_adc_banks,ctl_msg.timer_period); */

    sleep(1);                                                               /* wait for driver to read our struct */

    return 0;
}
