/*======================================================================

    das16s-config.h    v0.03    Apr 26 2000
    Configuration defs for das16s.c PCMCIA real-time client driver.
    Include this for reading and writing to the driver.

    for Real Time Linux kernels only, at present.
    see http://www.rtlinux.org

    by Steve Rosenbluth    

    Copyright (C) 2000 Jim Henson's Creature Shop

    The contents of this file are subject to the Mozilla Public
    License Version 1.0 (the "License"); you may not use this file
    except in compliance with the License. You may obtain a copy of
    the License at http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS
    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
    implied. See the License for the specific language governing
    rights and limitations under the License.
 
    Copyright(C) 2000 Jim Henson's Creature Shop

    thanks to Computer Boards, Inc. for providing register documentation
======================================================================*/
/*  epson dot-matrix printer condense code */

#ifndef _DAS16S_CONFIG_H_   /* include protection */
#define _DAS16S_CONFIG_H_

#define STOP_ACQUISITION        0   /* command flags sent to daq driver in control struct */
#define START_ACQUISITION       1
#define CONFIGURE_ACQUISITION   2
#define BURST_ACQUISITION       3
#define CONTIN_ACQUISITION      4
#define DIGITAL_OUTPUT          5
#define QUERY_ACQUISITION       6
#define BI_TEN_VOLT             8
#define BI_FIVE_VOLT            0
#define BI_TWO_VOLT             1
#define BI_ONE_VOLT             2
#define UNI_TEN_VOLT            4
#define UNI_FIVE_VOLT           5
#define UNI_TWO_VOLT            6
#define UNI_ONE_VOLT            7
#define CLR_2_LSB_INT32         0xFFFC /* AND mask to clear two low bits of an int32 */
#define MAX_SAMPLE_RATE_HZ      100000     /* maximum supported conversion rate */
#define INTERNAL_CLOCK_HZ       10000000    /* card has a 10Mhz internal clock */
#define MAX_BANKS               4       /* max number of banks of 16 samples we can get in burst mode */ 
#define MAX_CHANNELS            16      /* can only sample 1-16 channels on this card */
#define BTC_NUMSYS              0       /* Binary Twos Complement. F800 to 0000 is -Vmin to 0V, 0000 to 7FF is 0V to +Vmax */
#define BOB_NUMSYS              1       /* Bipolar Offset Binary. 0 to 4095 is -Vmin to +Vmax */
#ifndef TRUE
    #define TRUE                1
#endif
#ifndef FALSE
    #define FALSE               0
#endif
#define NONE                    -1

#define DAS16S_IOC_MAGIC        0xAD                                            /* magic number for our ioctl, not registered w/ Linus ! */
#define DAS16S_IOC_CONFIGURE        _IOW(DAS16S_IOC_MAGIC,1,ctl_msg_struct_p)
#define DAS16S_IOC_CLR_USAGE_CNT    _IO(DAS16S_IOC_MAGIC,2)

#define SAMPLE_HEADER_SIZE      3                                           /* we put this much of a header in each result packet */
#define SAMPLE_HEADER_MAGIC_1   0xCB                                        /* identifies begin of a result packet */
#define SAMPLE_HEADER_MAGIC_2   0xADC                                       /* identifies begin of a result packet */
#define HDR_MAGIC_OFFSET        0                                           /* place in result packet where magic num is found */
#define HDR_MUX_OFFSET          2                                           /* place in result packet where mux value is found */
#define DIG_IN_OFFSET           3                                           /* place in result packet where digital input data found */
#define ADC_DATA_OFFSET         4                                           /* place in result packet where analog input data found */
#define ADC_DATA_QTY            MAX_CHANNELS                                /* num adc samples per result packet */
#define DIG_DATA_QTY            1
#define RESULT_PACK_SIZE_B      (RESULT_PACK_SIZE_SI*2)                     /* number of bytes in a single result packet */
#define RESULT_PACK_SIZE_SI     (SAMPLE_HEADER_SIZE+DIG_DATA_QTY+ADC_DATA_QTY) /*20*/ /* number of short ints in a single result packet */
#define MAX_SAMPLE_PACKETS      32                                         /* driver (& card) holds this many sample packets (of 16) */
#define DAS_STOP                0
#define DAS_START               1
#define DAS_RESTART             2


/* Config struct sent to das16s driver. Values set to -1 will be ignored */
typedef struct {   /* message sent to driver through rt-fifo to change daq parameters */
    int command;    /* configure, start or stop conversions */
    int conversion_mode;  /* reset to default mode, continuous mode, burst mode, */
    int num_chan;   /* how many (of 16) channels to scan */
    int single_chan;    /* the channel to sample when in single chan mode */
    int num_mux_banks;  /* how many banks (of num_chan) to convert (4 max), using external multiplexer */
    int volt_range;    /* 10 volt, 5V, 2.5V, 1.25V bipolar or single ended - see constants */
/*    int result_numsys;*/  /* Binary Two's Complement, Bipolar Offset Binary */
    int error_correcting;   /* whether user wants to use hardware calibration constants on results: 1=yes, 0=no*/
    int adc_rate_divisor; /* configures ADC rate. divides down from 1MHz if aux_adc_rate_divisor is 10 */
    int aux_adc_rate_divisor; /* generates clock for adc_rate_divisor. divides down from 10MHz */
    /* 8 bit value to be written to the digital output port. */
    unsigned char dio_output_val;     /*NOTE: only 6 MSBs get written in multi-bank modes, 2 LSBs reserved for multiplexing */
    int num_dig_in_banks;   /* number of banks of digital inputs to get (4 max) */
} ctl_msg_struct_t;

#endif /* _DAS16S_CONFIG_H_ */
